@@ -1,11 +1,12 @@
 import time
-import logging
 import threading
 from transitions import Machine
 
+from State import State
 from PackageInfo import PackageInfo
 from ClientLogger import ClientLogger
 from DeviceApi import DeviceApi, Channel
+from CommandExecutor import CommandExecutor
 from TinkerAccessServerApi import TinkerAccessServerApi
 from ClientOptionParser import ClientOptionParser, ClientOption
 from UserRegistrationException import UserRegistrationException
@@ -16,14 +17,6 @@ training_mode_delay_seconds = 2
 logout_timer_interval_seconds = 1
 
 
-class State(object):
-    INITIALIZED = 'initialized'
-    IDLE = 'idle'
-    IN_USE = 'in_use'
-    IN_TRAINING = 'in_training'
-    TERMINATED = 'terminated'
-
-
 class Trigger(object):
     IDLE = 'idle'
     LOGIN = 'login'
@@ -124,7 +117,6 @@ class Client(Machine):
         except Exception as e:
             self.__handle_unexpected_exception()
             self.__ensure_idle()
-            raise e
 
         return self.__user_info is not None
 
@@ -313,7 +305,6 @@ class Client(Machine):
         except Exception as e:
             self.__handle_unexpected_exception()
             self.__prompt_for_trainer_badge()
-            raise e
 
         return self.__user_info is not None
 
@@ -357,11 +348,10 @@ class Client(Machine):
             self.__show_student_registered(1)
         except UserRegistrationException:
             self.__handle_user_registration_exception()
-            self.__show_unknown_user(2)
+            self.__show_invalid_user(2)
         except Exception as e:
             self.__handle_user_registration_exception()
             self.__handle_unexpected_exception()
-            raise e
         finally:
             self.__prompt_for_student_badge()
 
@@ -393,10 +383,10 @@ class Client(Machine):
         )
         time.sleep(delay)
 
-    def __show_unknown_user(self, delay=0):
+    def __show_invalid_user(self, delay=0):
         self.__device.write(
             Channel.LCD,
-            'UNKNOWN'.center(maximum_lcd_characters, ' '),
+            'INVALID'.center(maximum_lcd_characters, ' '),
             'USER...'.center(maximum_lcd_characters, ' ')
         )
         time.sleep(delay)
@@ -550,38 +540,49 @@ class Client(Machine):
     def run(opts, args):
         should_exit = False
         logger = ClientLogger.setup()
-        restart_delay = opts.get(ClientOption.RESTART_DELAY)
+        reboot_delay = opts.get(ClientOption.REBOOT_DELAY) * 60
+        reboot_on_error = opts.get(ClientOption.REBOOT_ON_ERROR)
 
-        while not should_exit:
-            try:
-                with DeviceApi(opts) as device, \
-                        Client(device) as client:
+        try:
+            with DeviceApi(opts) as device, Client(device) as client:
 
-                    device.on(
-                        Channel.SERIAL,
-                        direction=device.GPIO.IN,
-                        call_back=client.handle_badge_code
-                    )
+                device.on(
+                    Channel.SERIAL,
+                    direction=device.GPIO.IN,
+                    call_back=client.handle_badge_code
+                )
 
-                    device.on(
-                        Channel.PIN,
-                        pin=opts.get(ClientOption.PIN_LOGOUT),
-                        direction=device.GPIO.RISING,
-                        call_back=client.logout
-                    )
+                device.on(
+                    Channel.PIN,
+                    pin=opts.get(ClientOption.PIN_LOGOUT),
+                    direction=device.GPIO.RISING,
+                    call_back=client.logout
+                )
 
-                    client.idle()
-                    while not client.is_terminated():
-                        logger.debug('%s is waiting...', PackageInfo.pip_package_name)
-                        client.wait()
-                    should_exit = True
-
-            except (KeyboardInterrupt, SystemExit):
+                client.idle()
+                while not client.is_terminated():
+                    logger.debug('%s is waiting...', PackageInfo.pip_package_name)
+                    client.wait()
                 should_exit = True
 
-            except Exception as e:
-                logger.exception(e)
-                logger.debug('Retrying in %s seconds...', restart_delay)
-                time.sleep(restart_delay)
+        except (KeyboardInterrupt, SystemExit):
+            should_exit = True
+
+        except Exception as e:
+            logger.exception(e)
+
+            if reboot_on_error:
+
+                # reboot is only supported on Raspberry PI devices
+                # noinspection PyBroadException
+                try:
+                    # noinspection PyUnresolvedReferences
+                    import RPi.GPIO
+                    logger.error('Rebooting in %s minutes...', reboot_delay / 60)
+                    CommandExecutor().execute_commands([
+                        'sleep {0}s'.format(reboot_delay),
+                        'reboot now'
+                    ])
+                except Exception:
+                    pass
 
-        logging.shutdown()
