@@ -10,9 +10,11 @@ class ClientOption:
     PIN_LED_RED = 'pin_led_red'
     STATUS_FILE = 'status_file'
     PIN_LED_BLUE = 'pin_led_blue'
-    RESTART_DELAY = 'restart_delay'
+    REBOOT_DELAY = 'reboot_delay'
+    FORCE_UPDATE = 'force_update'
     PIN_LED_GREEN = 'pin_led_green'
     SERVER_ADDRESS = 'server_address'
+    REBOOT_ON_ERROR = 'reboot_on_error'
     PIN_POWER_RELAY = 'pin_power_relay'
     SERIAL_PORT_NAME = 'serial_port_name'
     SERIAL_PORT_SPEED = 'serial_port_speed'
