@@ -13,8 +13,10 @@
 # between RPi's modules (i.e. RFID reader, LCD, power relays etc..) and the remote tinker_access_server.
 ### END INIT INFO
 
+import os
 import sys
 import logging
+from PackageInfo import PackageInfo
 from ClientLogger import ClientLogger
 from ClientDaemon import ClientDaemon
 from CommandHandler import CommandHandler
@@ -22,6 +24,13 @@ from ClientOptionParser import ClientOptionParser, Command
 
 
 def run():
+    if os.geteuid() != 0:
+        sys.stdout.write(
+            'You need to have root privileges to run {0} commands.\n'
+            'Please try again, this time using \'sudo\'.\n'.format(PackageInfo.pip_package_name))
+        sys.stdout.flush()
+        sys.exit(1)
+
     logger = ClientLogger.setup()
     (opts, args) = ClientOptionParser().parse_args()
 
@@ -30,6 +39,7 @@ def run():
             handler.on(Command.STOP, ClientDaemon.stop)
             handler.on(Command.START, ClientDaemon.start)
             handler.on(Command.STATUS, ClientDaemon.status)
+            handler.on(Command.UPDATE, ClientDaemon.update)
             handler.on(Command.RESTART, ClientDaemon.restart)
             return handler.handle_command()
 
